/*
 *   Copyright (C) 2014 - 2016 by Ivan Cukic <ivan.cukic(at)kde.org>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) version 3, or any
 *   later version accepted by the membership of KDE e.V. (or its
 *   successor approved by the membership of KDE e.V.), which shall
 *   act as a proxy defined in Section 6 of version 3 of the license.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library.
 *   If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ACTIVITIES_MAINTHREADEXECUTOR_P
#define ACTIVITIES_MAINTHREADEXECUTOR_P

#include <functional>

#include <QObject>

namespace KActivities {

namespace detail {
    class MainThreadExecutor: public QObject {
        Q_OBJECT

    public:
        MainThreadExecutor(std::function<void()> &&f);

        Q_INVOKABLE void start();

    private:
        std::function<void()> m_function;
    };
} // namespace detail

void runInMainThread(std::function<void()> &&f);

} // namespace KActivities

#endif // ACTIVITIES_MAINTHREADEXECUTOR_P
